@@ -1,11 +1,14 @@
 // Copyright 2018-2022 the Deno authors. All rights reserved. MIT license.
 
 use super::analysis;
+use super::cache;
 use super::client::Client;
 use super::config::ConfigSnapshot;
 use super::documents;
+use super::documents::Document;
 use super::documents::Documents;
 use super::language_server;
+use super::language_server::StateSnapshot;
 use super::performance::Performance;
 use super::tsc;
 use super::tsc::TsServer;
@@ -34,6 +37,8 @@ use tokio::time::Duration;
 use tokio::time::Instant;
 use tokio_util::sync::CancellationToken;
 
+pub(crate) type SnapshotForDiagnostics =
+  (Arc<StateSnapshot>, Arc<ConfigSnapshot>, Option<LintConfig>);
 pub type DiagnosticRecord =
   (ModuleSpecifier, Option<i32>, Vec<lsp::Diagnostic>);
 pub type DiagnosticVec = Vec<DiagnosticRecord>;
@@ -89,10 +94,53 @@ impl DiagnosticsPublisher {
   }
 }
 
+#[derive(Clone, Default, Debug)]
+struct TsDiagnosticsStore(Arc<deno_core::parking_lot::Mutex<DiagnosticMap>>);
+
+impl TsDiagnosticsStore {
+  pub fn get(
+    &self,
+    specifier: &ModuleSpecifier,
+    document_version: Option<i32>,
+  ) -> Vec<lsp::Diagnostic> {
+    let ts_diagnostics = self.0.lock();
+    if let Some((diagnostics_doc_version, diagnostics)) =
+      ts_diagnostics.get(specifier)
+    {
+      // only get the diagnostics if they're up to date
+      if document_version == *diagnostics_doc_version {
+        return diagnostics.clone();
+      }
+    }
+    Vec::new()
+  }
+
+  pub fn invalidate(&self, specifiers: &[ModuleSpecifier]) {
+    let mut ts_diagnostics = self.0.lock();
+    for specifier in specifiers {
+      ts_diagnostics.remove(specifier);
+    }
+  }
+
+  pub fn invalidate_all(&self) {
+    self.0.lock().clear();
+  }
+
+  fn update(&self, diagnostics: &DiagnosticVec) {
+    let mut stored_ts_diagnostics = self.0.lock();
+    *stored_ts_diagnostics = diagnostics
+      .iter()
+      .map(|(specifier, version, diagnostics)| {
+        (specifier.clone(), (*version, diagnostics.clone()))
+      })
+      .collect();
+  }
+}
+
 #[derive(Debug)]
 pub(crate) struct DiagnosticsServer {
-  channel: Option<mpsc::UnboundedSender<()>>,
-  ts_diagnostics: Arc<Mutex<DiagnosticMap>>,
+  channel: Option<mpsc::UnboundedSender<SnapshotForDiagnostics>>,
+  ts_diagnostics: TsDiagnosticsStore,
   client: Client,
   performance: Arc<Performance>,
   ts_server: Arc<TsServer>,
@@ -113,44 +161,28 @@ impl DiagnosticsServer {
     }
   }
 
-  pub(crate) async fn get_ts_diagnostics(
+  pub(crate) fn get_ts_diagnostics(
     &self,
     specifier: &ModuleSpecifier,
     document_version: Option<i32>,
   ) -> Vec<lsp::Diagnostic> {
-    let ts_diagnostics = self.ts_diagnostics.lock().await;
-    if let Some((diagnostics_doc_version, diagnostics)) =
-      ts_diagnostics.get(specifier)
-    {
-      // only get the diagnostics if they're up to date
-      if document_version == *diagnostics_doc_version {
-        return diagnostics.clone();
-      }
-    }
-    Vec::new()
+    self.ts_diagnostics.get(specifier, document_version)
   }
 
-  pub(crate) async fn invalidate(&self, specifiers: Vec<ModuleSpecifier>) {
-    let mut ts_diagnostics = self.ts_diagnostics.lock().await;
-    for specifier in &specifiers {
-      ts_diagnostics.remove(specifier);
-    }
+  pub(crate) fn invalidate(&self, specifiers: &[ModuleSpecifier]) {
+    self.ts_diagnostics.invalidate(specifiers);
   }
 
-  pub(crate) async fn invalidate_all(&self) {
-    let mut ts_diagnostics = self.ts_diagnostics.lock().await;
-    ts_diagnostics.clear();
+  pub(crate) fn invalidate_all(&self) {
+    self.ts_diagnostics.invalidate_all();
   }
 
-  pub(crate) fn start(
-    &mut self,
-    language_server: Arc<Mutex<language_server::Inner>>,
-  ) {
-    let (tx, mut rx) = mpsc::unbounded_channel::<()>();
+  pub(crate) fn start(&mut self) {
+    let (tx, mut rx) = mpsc::unbounded_channel::<SnapshotForDiagnostics>();
     self.channel = Some(tx);
     let client = self.client.clone();
     let performance = self.performance.clone();
-    let stored_ts_diagnostics = self.ts_diagnostics.clone();
+    let ts_diagnostics_store = self.ts_diagnostics.clone();
     let ts_server = self.ts_server.clone();
 
     let _join_handle = thread::spawn(move || {
@@ -167,28 +199,19 @@ impl DiagnosticsServer {
           match rx.recv().await {
             // channel has closed
             None => break,
-            Some(()) => {
+            Some((snapshot, config, maybe_lint_config)) => {
               // cancel the previous run
               token.cancel();
               token = CancellationToken::new();
               diagnostics_publisher.clear().await;
 
-              let (snapshot, config, maybe_lint_config) = {
-                let language_server = language_server.lock().await;
-                (
-                  language_server.snapshot(),
-                  language_server.config.snapshot(),
-                  language_server.maybe_lint_config.clone(),
-                )
-              };
-
               let previous_ts_handle = ts_handle.take();
               ts_handle = Some(tokio::spawn({
                 let performance = performance.clone();
                 let diagnostics_publisher = diagnostics_publisher.clone();
                 let ts_server = ts_server.clone();
                 let token = token.clone();
-                let stored_ts_diagnostics = stored_ts_diagnostics.clone();
+                let ts_diagnostics_store = ts_diagnostics_store.clone();
                 let snapshot = snapshot.clone();
                 let config = config.clone();
                 async move {
@@ -211,29 +234,20 @@ impl DiagnosticsServer {
 
                   let mark =
                     performance.mark("update_diagnostics_ts", None::<()>);
-                  let diagnostics =
-                    generate_ts_diagnostics(snapshot.clone(), &ts_server)
-                      .await
-                      .map_err(|err| {
-                        error!(
-                          "Error generating TypeScript diagnostics: {}",
-                          err
-                        );
-                      })
-                      .unwrap_or_default();
+                  let diagnostics = generate_ts_diagnostics(
+                    snapshot.clone(),
+                    &config,
+                    &ts_server,
+                    token.clone(),
+                  )
+                  .await
+                  .map_err(|err| {
+                    error!("Error generating TypeScript diagnostics: {}", err);
+                  })
+                  .unwrap_or_default();
 
                   if !token.is_cancelled() {
-                    {
-                      let mut stored_ts_diagnostics =
-                        stored_ts_diagnostics.lock().await;
-                      *stored_ts_diagnostics = diagnostics
-                        .iter()
-                        .map(|(specifier, version, diagnostics)| {
-                          (specifier.clone(), (*version, diagnostics.clone()))
-                        })
-                        .collect();
-                    }
-
+                    ts_diagnostics_store.update(&diagnostics);
                     diagnostics_publisher.publish(diagnostics, &token).await;
 
                     if !token.is_cancelled() {
@@ -257,8 +271,8 @@ impl DiagnosticsServer {
                   let mark =
                     performance.mark("update_diagnostics_deps", None::<()>);
                   let diagnostics = generate_deps_diagnostics(
-                    snapshot.clone(),
-                    config.clone(),
+                    &snapshot,
+                    &config,
                     token.clone(),
                   )
                   .await;
@@ -306,9 +320,15 @@ impl DiagnosticsServer {
     });
   }
 
-  pub(crate) fn update(&self) -> Result<(), AnyError> {
+  pub(crate) fn update(
+    &self,
+    message: SnapshotForDiagnostics,
+  ) -> Result<(), AnyError> {
+    // todo(dsherret): instead of queuing up messages, it would be better to
+    // instead only store the latest message (ex. maybe using a
+    // tokio::sync::watch::channel)
     if let Some(tx) = &self.channel {
-      tx.send(()).map_err(|err| err.into())
+      tx.send(message).map_err(|err| err.into())
     } else {
       Err(anyhow!("diagnostics server not started"))
     }
@@ -439,49 +459,60 @@ async fn generate_lint_diagnostics(
       }
 
       let version = document.maybe_lsp_version();
-      let is_allowed = match &maybe_lint_config {
-        Some(lint_config) => {
-          lint_config.files.matches_specifier(document.specifier())
-        }
-        None => true,
-      };
-      let diagnostics = if is_allowed {
-        match document.maybe_parsed_source() {
-          Some(Ok(parsed_source)) => {
-            if let Ok(references) = analysis::get_lint_references(
-              &parsed_source,
-              maybe_lint_config.as_ref(),
-            ) {
-              references
-                .into_iter()
-                .map(|r| r.to_diagnostic())
-                .collect::<Vec<_>>()
-            } else {
-              Vec::new()
-            }
-          }
-          Some(Err(_)) => Vec::new(),
-          None => {
-            error!("Missing file contents for: {}", document.specifier());
-            Vec::new()
-          }
-        }
-      } else {
-        Vec::new()
-      };
       diagnostics_vec.push((
         document.specifier().clone(),
         version,
-        diagnostics,
+        generate_document_lint_diagnostics(
+          config,
+          &maybe_lint_config,
+          &document,
+        ),
       ));
     }
   }
   diagnostics_vec
 }
 
+fn generate_document_lint_diagnostics(
+  config: &ConfigSnapshot,
+  maybe_lint_config: &Option<LintConfig>,
+  document: &Document,
+) -> Vec<lsp::Diagnostic> {
+  if !config.specifier_enabled(document.specifier()) {
+    return Vec::new();
+  }
+  if let Some(lint_config) = &maybe_lint_config {
+    if !lint_config.files.matches_specifier(document.specifier()) {
+      return Vec::new();
+    }
+  }
+  match document.maybe_parsed_source() {
+    Some(Ok(parsed_source)) => {
+      if let Ok(references) = analysis::get_lint_references(
+        &parsed_source,
+        maybe_lint_config.as_ref(),
+      ) {
+        references
+          .into_iter()
+          .map(|r| r.to_diagnostic())
+          .collect::<Vec<_>>()
+      } else {
+        Vec::new()
+      }
+    }
+    Some(Err(_)) => Vec::new(),
+    None => {
+      error!("Missing file contents for: {}", document.specifier());
+      Vec::new()
+    }
+  }
+}
+
 async fn generate_ts_diagnostics(
   snapshot: Arc<language_server::StateSnapshot>,
+  config: &ConfigSnapshot,
   ts_server: &tsc::TsServer,
+  token: CancellationToken,
 ) -> Result<DiagnosticVec, AnyError> {
   let mut diagnostics_vec = Vec::new();
   let specifiers = snapshot
@@ -490,23 +521,43 @@ async fn generate_ts_diagnostics(
     .iter()
     .map(|d| d.specifier().clone())
     .collect::<Vec<_>>();
-  if !specifiers.is_empty() {
-    let req = tsc::RequestMethod::GetDiagnostics(specifiers);
-    let ts_diagnostics_map: TsDiagnosticsMap =
-      ts_server.request(snapshot.clone(), req).await?;
-    for (specifier_str, ts_diagnostics) in ts_diagnostics_map {
-      let specifier = resolve_url(&specifier_str)?;
-      let version = snapshot
-        .documents
-        .get(&specifier)
-        .map(|d| d.maybe_lsp_version())
-        .flatten();
-      diagnostics_vec.push((
-        specifier,
-        version,
-        ts_json_to_diagnostics(ts_diagnostics),
-      ));
-    }
+  let (enabled_specifiers, disabled_specifiers) = specifiers
+    .iter()
+    .cloned()
+    .partition::<Vec<_>, _>(|s| config.specifier_enabled(s));
+  let ts_diagnostics_map: TsDiagnosticsMap = if !enabled_specifiers.is_empty() {
+    let req = tsc::RequestMethod::GetDiagnostics(enabled_specifiers);
+    ts_server
+      .request_with_cancellation(snapshot.clone(), req, token)
+      .await?
+  } else {
+    Default::default()
+  };
+  for (specifier_str, ts_json_diagnostics) in ts_diagnostics_map {
+    let specifier = resolve_url(&specifier_str)?;
+    let version = snapshot
+      .documents
+      .get(&specifier)
+      .map(|d| d.maybe_lsp_version())
+      .flatten();
+    // check if the specifier is enabled again just in case TS returns us
+    // diagnostics for a disabled specifier
+    let ts_diagnostics = if config.specifier_enabled(&specifier) {
+      ts_json_to_diagnostics(ts_json_diagnostics)
+    } else {
+      Vec::new()
+    };
+    diagnostics_vec.push((specifier, version, ts_diagnostics));
+  }
+  // add an empty diagnostic publish for disabled specifiers in order
+  // to clear those diagnostics if they exist
+  for specifier in disabled_specifiers {
+    let version = snapshot
+      .documents
+      .get(&specifier)
+      .map(|d| d.maybe_lsp_version())
+      .flatten();
+    diagnostics_vec.push((specifier, version, Vec::new()));
   }
   Ok(diagnostics_vec)
 }
@@ -541,14 +592,17 @@ fn resolution_error_as_code(
 fn diagnose_dependency(
   diagnostics: &mut Vec<lsp::Diagnostic>,
   documents: &Documents,
+  cache_metadata: &cache::CacheMetadata,
   resolved: &deno_graph::Resolved,
   is_dynamic: bool,
   maybe_assert_type: Option<&str>,
 ) {
   match resolved {
     Some(Ok((specifier, range))) => {
-      if let Some(doc) = documents.get(specifier) {
-        if let Some(message) = doc.maybe_warning() {
+      if let Some(metadata) = cache_metadata.get(specifier) {
+        if let Some(message) =
+          metadata.get(&cache::MetadataKey::Warning).cloned()
+        {
           diagnostics.push(lsp::Diagnostic {
             range: documents::to_lsp_range(range),
             severity: Some(lsp::DiagnosticSeverity::WARNING),
@@ -556,8 +610,10 @@ fn diagnose_dependency(
             source: Some("deno".to_string()),
             message,
             ..Default::default()
-          })
+          });
         }
+      }
+      if let Some(doc) = documents.get(specifier) {
         if doc.media_type() == MediaType::Json {
           match maybe_assert_type {
             // The module has the correct assertion type, no diagnostic
@@ -619,8 +675,8 @@ fn diagnose_dependency(
 /// Generate diagnostics for dependencies of a module, attempting to resolve
 /// dependencies on the local file system or in the DENO_DIR cache.
 async fn generate_deps_diagnostics(
-  snapshot: Arc<language_server::StateSnapshot>,
-  config: Arc<ConfigSnapshot>,
+  snapshot: &language_server::StateSnapshot,
+  config: &ConfigSnapshot,
   token: CancellationToken,
 ) -> DiagnosticVec {
   let mut diagnostics_vec = Vec::new();
@@ -629,25 +685,26 @@ async fn generate_deps_diagnostics(
     if token.is_cancelled() {
       break;
     }
-    if !config.specifier_enabled(document.specifier()) {
-      continue;
-    }
     let mut diagnostics = Vec::new();
-    for (_, dependency) in document.dependencies() {
-      diagnose_dependency(
-        &mut diagnostics,
-        &snapshot.documents,
-        &dependency.maybe_code,
-        dependency.is_dynamic,
-        dependency.maybe_assert_type.as_deref(),
-      );
-      diagnose_dependency(
-        &mut diagnostics,
-        &snapshot.documents,
-        &dependency.maybe_type,
-        dependency.is_dynamic,
-        dependency.maybe_assert_type.as_deref(),
-      );
+    if config.specifier_enabled(document.specifier()) {
+      for (_, dependency) in document.dependencies() {
+        diagnose_dependency(
+          &mut diagnostics,
+          &snapshot.documents,
+          &snapshot.cache_metadata,
+          &dependency.maybe_code,
+          dependency.is_dynamic,
+          dependency.maybe_assert_type.as_deref(),
+        );
+        diagnose_dependency(
+          &mut diagnostics,
+          &snapshot.documents,
+          &snapshot.cache_metadata,
+          &dependency.maybe_type,
+          dependency.is_dynamic,
+          dependency.maybe_assert_type.as_deref(),
+        );
+      }
     }
     diagnostics_vec.push((
       document.specifier().clone(),
@@ -664,6 +721,7 @@ mod tests {
   use super::*;
   use crate::lsp::config::ConfigSnapshot;
   use crate::lsp::config::Settings;
+  use crate::lsp::config::SpecifierSettings;
   use crate::lsp::config::WorkspaceSettings;
   use crate::lsp::documents::LanguageId;
   use crate::lsp::language_server::StateSnapshot;
@@ -708,31 +766,126 @@ mod tests {
 
   fn setup(
     sources: &[(&str, &str, i32, LanguageId)],
-  ) -> (StateSnapshot, PathBuf, ConfigSnapshot) {
+  ) -> (StateSnapshot, PathBuf) {
     let temp_dir = TempDir::new().expect("could not create temp dir");
     let location = temp_dir.path().join("deps");
     let state_snapshot = mock_state_snapshot(sources, &location);
-    let config = mock_config();
-    (state_snapshot, location, config)
+    (state_snapshot, location)
   }
 
   #[tokio::test]
-  async fn test_generate_lint_diagnostics() {
-    let (snapshot, _, config) = setup(&[(
+  async fn test_enabled_then_disabled_specifier() {
+    let specifier = ModuleSpecifier::parse("file:///a.ts").unwrap();
+    let (snapshot, _) = setup(&[(
       "file:///a.ts",
       r#"import * as b from "./b.ts";
-
-let a = "a";
-console.log(a);
+let a: any = "a";
+let c: number = "a";
 "#,
       1,
       LanguageId::TypeScript,
     )]);
+    let snapshot = Arc::new(snapshot);
+    let ts_server = TsServer::new(Default::default());
+
+    // test enabled
+    {
+      let enabled_config = mock_config();
+      let diagnostics = generate_lint_diagnostics(
+        &snapshot,
+        &enabled_config,
+        None,
+        Default::default(),
+      )
+      .await;
+      assert_eq!(get_diagnostics_for_single(diagnostics).len(), 6);
+      let diagnostics = generate_ts_diagnostics(
+        snapshot.clone(),
+        &enabled_config,
+        &ts_server,
+        Default::default(),
+      )
+      .await
+      .unwrap();
+      assert_eq!(get_diagnostics_for_single(diagnostics).len(), 4);
+      let diagnostics = generate_deps_diagnostics(
+        &snapshot,
+        &enabled_config,
+        Default::default(),
+      )
+      .await;
+      assert_eq!(get_diagnostics_for_single(diagnostics).len(), 1);
+    }
+
+    // now test disabled specifier
+    {
+      let mut disabled_config = mock_config();
+      disabled_config.settings.specifiers.insert(
+        specifier.clone(),
+        (
+          specifier.clone(),
+          SpecifierSettings {
+            enable: false,
+            code_lens: Default::default(),
+          },
+        ),
+      );
+
+      let diagnostics = generate_lint_diagnostics(
+        &snapshot,
+        &disabled_config,
+        None,
+        Default::default(),
+      )
+      .await;
+      assert_eq!(get_diagnostics_for_single(diagnostics).len(), 0);
+      let diagnostics = generate_ts_diagnostics(
+        snapshot.clone(),
+        &disabled_config,
+        &ts_server,
+        Default::default(),
+      )
+      .await
+      .unwrap();
+      assert_eq!(get_diagnostics_for_single(diagnostics).len(), 0);
+      let diagnostics = generate_deps_diagnostics(
+        &snapshot,
+        &disabled_config,
+        Default::default(),
+      )
+      .await;
+      assert_eq!(get_diagnostics_for_single(diagnostics).len(), 0);
+    }
+  }
+
+  fn get_diagnostics_for_single(
+    diagnostic_vec: DiagnosticVec,
+  ) -> Vec<lsp::Diagnostic> {
+    assert_eq!(diagnostic_vec.len(), 1);
+    let (_, _, diagnostics) = diagnostic_vec.into_iter().next().unwrap();
+    diagnostics
+  }
+
+  #[tokio::test]
+  async fn test_cancelled_ts_diagnostics_request() {
+    let specifier = ModuleSpecifier::parse("file:///a.ts").unwrap();
+    let (snapshot, _) = setup(&[(
+      "file:///a.ts",
+      r#"export let a: string = 5;"#,
+      1,
+      LanguageId::TypeScript,
+    )]);
+    let snapshot = Arc::new(snapshot);
+    let ts_server = TsServer::new(Default::default());
+
+    let config = mock_config();
+    let token = CancellationToken::new();
+    token.cancel();
     let diagnostics =
-      generate_lint_diagnostics(&snapshot, &config, None, Default::default())
-        .await;
-    assert_eq!(diagnostics.len(), 1);
-    let (_, _, diagnostics) = &diagnostics[0];
-    assert_eq!(diagnostics.len(), 2);
+      generate_ts_diagnostics(snapshot.clone(), &config, &ts_server, token)
+        .await
+        .unwrap();
+    // should be none because it's cancelled
+    assert_eq!(diagnostics.len(), 0);
   }
 }
